"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
   Copyright 2016 Yuki KAN

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
const fs = require("fs");
const http = require("http");
const querystring = require("querystring");
const yaml = require("js-yaml");
const pkg = require("../package.json");
const spec = yaml.safeLoad(fs.readFileSync(__dirname + "/../api.yml", "utf8"));
class Client {
    constructor() {
        this.basePath = spec.basePath;
        this.docsPath = "/docs";
        /** positive integer */
        this.priority = 0;
        this.host = "";
        this.port = 40772;
        this.socketPath = process.platform === "win32" ? "\\\\.\\pipe\\mirakurun" : "/var/run/mirakurun.sock";
        /** provide User-Agent string to identify client. */
        this.userAgent = "";
        this._userAgent = `MirakurunClient/${pkg.version} Node/${process.version} (${process.platform})`;
    }
    request(method, path, option = {}) {
        return new Promise((resolve, reject) => {
            this._httpRequest(method, path, option).then(res => {
                const ret = {
                    status: res.statusCode,
                    statusText: res.statusMessage,
                    contentType: res.headers["content-type"].split(";")[0],
                    headers: res.headers,
                    isSuccess: (res.statusCode >= 200 && res.statusCode <= 202)
                };
                const chunks = [];
                res.on("data", chunk => chunks.push(chunk));
                res.on("end", () => {
                    const buffer = Buffer.concat(chunks);
                    if (ret.contentType === "application/json") {
                        ret.body = JSON.parse(buffer.toString("utf8"));
                    }
                    else if (ret.contentType === "text/plain") {
                        ret.body = buffer.toString("utf8");
                    }
                    else {
                        ret.body = buffer;
                    }
                    if (ret.isSuccess === true) {
                        resolve(ret);
                    }
                    else {
                        reject(ret);
                    }
                });
            }, err => {
                const ret = {
                    status: -1,
                    statusText: "Request Failure",
                    contentType: "",
                    headers: {},
                    isSuccess: false,
                    body: err
                };
                reject(ret);
            });
        });
    }
    call(operationId, param = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._docs) {
                yield this._getDocs();
            }
            let path;
            let method;
            let parameters;
            let operation;
            for (path in this._docs.paths) {
                const p = this._docs.paths[path];
                if (p.post && p.post.operationId === operationId) {
                    method = "POST";
                    parameters = [...p.parameters, ...(p.post.parameters || [])];
                    operation = p.post;
                    break;
                }
                if (p.get && p.get.operationId === operationId) {
                    method = "GET";
                    parameters = [...p.parameters, ...(p.get.parameters || [])];
                    operation = p.get;
                    break;
                }
                if (p.put && p.put.operationId === operationId) {
                    method = "PUT";
                    parameters = [...p.parameters, ...(p.put.parameters || [])];
                    operation = p.put;
                    break;
                }
                if (p.delete && p.delete.operationId === operationId) {
                    method = "DELETE";
                    parameters = [...p.parameters, ...(p.delete.parameters || [])];
                    operation = p.delete;
                    break;
                }
            }
            if (!operation) {
                throw new Error(`operationId "${operationId}" is not found.`);
            }
            const option = {
                headers: {},
                query: {}
            };
            for (const p of parameters) {
                if (param[p.name] === undefined || param[p.name] === null) {
                    if (p.required) {
                        throw new Error(`Required parameter "${p.name}" is undefined.`);
                    }
                    continue;
                }
                if (p.in === "path") {
                    path = path.replace(`{${p.name}}`, param[p.name]);
                }
                else if (p.in === "header") {
                    option.headers[p.name] = param[p.name];
                }
                else if (p.in === "query") {
                    option.query[p.name] = param[p.name];
                }
                else if (p.in === "body" && p.name === "body") {
                    option.body = param.body;
                }
            }
            if (operation.tags.indexOf("stream") !== -1) {
                return yield this._requestStream(method, path, option);
            }
            return yield this.request(method, path, option);
        });
    }
    getChannels(query) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getChannels", query);
            return res.body;
        });
    }
    getChannelsByType(type, query) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getChannelsByType", Object.assign({ type }, query));
            return res.body;
        });
    }
    getChannel(type, channel) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getChannel", { type, channel });
            return res.body;
        });
    }
    getServicesByChannel(type, channel) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getServicesByChannel", { type, channel });
            return res.body;
        });
    }
    getServiceByChannel(type, channel, sid) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getServiceByChannel", { type, channel, sid });
            return res.body;
        });
    }
    getServiceStreamByChannel(type, channel, sid, decode, priority) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.call("getServiceStreamByChannel", { type, channel, sid, "decode": decode ? 1 : 0, "X-Mirakurun-Priority": priority });
        });
    }
    getChannelStream(type, channel, decode, priority) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.call("getChannelStream", { type, channel, "decode": decode ? 1 : 0, "X-Mirakurun-Priority": priority });
        });
    }
    getPrograms(query) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getPrograms", query);
            return res.body;
        });
    }
    getProgram(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getProgram", { id });
            return res.body;
        });
    }
    getProgramStream(id, decode, priority) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.call("getProgramStream", { id, "decode": decode ? 1 : 0, "X-Mirakurun-Priority": priority });
        });
    }
    getServices(query) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getServices", query);
            return res.body;
        });
    }
    getService(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getService", { id });
            return res.body;
        });
    }
    getLogoImage(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getLogoImage", { id });
            return res.body;
        });
    }
    getServiceStream(id, decode, priority) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.call("getServiceStream", { id, "decode": decode ? 1 : 0, "X-Mirakurun-Priority": priority });
        });
    }
    getTuners() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getTuners");
            return res.body;
        });
    }
    getTuner(index) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getTuner", { index });
            return res.body;
        });
    }
    getTunerProcess(index) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getTunerProcess", { index });
            return res.body;
        });
    }
    killTunerProcess(index) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("killTunerProcess", { index });
            return res.body;
        });
    }
    getEvents() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getEvents");
            return res.body;
        });
    }
    getEventsStream(query) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.call("getEventsStream", query);
        });
    }
    getChannelsConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getChannelsConfig");
            return res.body;
        });
    }
    updateChannelsConfig(channels) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("updateChannelsConfig", { body: channels });
            return res.body;
        });
    }
    channelScan(option) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.call("channelScan", option);
        });
    }
    getServerConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getServerConfig");
            return res.body;
        });
    }
    updateServerConfig(server) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("updateServerConfig", { body: server });
            return res.body;
        });
    }
    getTunersConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getTunersConfig");
            return res.body;
        });
    }
    updateTunersConfig(tuners) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("updateTunersConfig", { body: tuners });
            return res.body;
        });
    }
    getLog() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getLog");
            return res.body;
        });
    }
    getLogStream() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.call("getLogStream");
        });
    }
    checkVersion() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("checkVersion");
            return res.body;
        });
    }
    updateVersion(force) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.call("updateVersion", { force });
        });
    }
    getStatus() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("getStatus");
            return res.body;
        });
    }
    restart() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.call("restart");
            return res.body;
        });
    }
    _httpRequest(method, path, option = {}) {
        const opt = {
            method: method,
            path: this.basePath + path,
            headers: option.headers || {},
            agent: this.agent
        };
        if (this.host === "") {
            opt.socketPath = this.socketPath;
        }
        else {
            opt.host = this.host;
            opt.port = this.port;
        }
        if (this.userAgent === "") {
            opt.headers["User-Agent"] = this._userAgent;
        }
        else {
            opt.headers["User-Agent"] = this.userAgent + " " + this._userAgent;
        }
        if (option.priority === undefined) {
            option.priority = this.priority;
        }
        opt.headers["X-Mirakurun-Priority"] = option.priority.toString(10);
        if (typeof option.query === "object") {
            opt.path += "?" + querystring.stringify(option.query);
        }
        if (typeof option.body === "object") {
            opt.headers["Content-Type"] = "application/json; charset=utf-8";
            option.body = JSON.stringify(option.body);
        }
        return new Promise((resolve, reject) => {
            const req = http.request(opt, res => {
                if (res.statusCode > 300 && res.statusCode < 400 && res.headers.location) {
                    if (/^\//.test(res.headers.location) === false) {
                        reject(new Error(`Redirecting location "${res.headers.location}" isn't supported.`));
                        return;
                    }
                    this._httpRequest(method, res.headers.location, option)
                        .then(resolve, reject);
                    return;
                }
                resolve(res);
            });
            req.on("error", reject);
            // write request body
            if (typeof option.body === "string") {
                req.write(option.body + "\n");
            }
            req.end();
        });
    }
    _requestStream(method, path, option = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this._httpRequest(method, path, option);
            if (res.statusCode >= 200 && res.statusCode <= 202) {
                return res;
            }
            else {
                if (res.statusCode) {
                    throw new Error(`Bad status respond (${res.statusCode} ${res.statusMessage}).`);
                }
                throw res;
            }
        });
    }
    _getDocs() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.request("GET", this.docsPath);
            if (res.isSuccess !== true) {
                throw new Error(`Failed to get "${this.docsPath}".`);
            }
            this._docs = res.body;
        });
    }
}
exports.default = Client;

//# sourceMappingURL=client.js.map
